"""
__version__.py
~~~~~~~~~~~~~~

Information about the current version of the clustermil package.
"""

__title__ = 'clustermil'
__description__ = 'clustermil - clustering based multiple instance learning'
__version__ = '0.2.0'
__author__ = 'Akimitsu Inoue'
__author_email__ = 'akimitsu.inoue@gmail.com'
__license__ = 'MIT'
__url__ = 'https://github.com/inoueakimitsu/clustermil'
